package hu.bme.mit.bookmanager;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

public class Book {

	private String author;
	private String title;
	private int pageNum;
	private boolean isRead;
	
	private PropertyChangeSupport changeSupport = new PropertyChangeSupport(
			this);

	public void addPropertyChangeListener(String propertyName,
			PropertyChangeListener listener) {
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(String propertyName,
			PropertyChangeListener listener) {
		changeSupport.removePropertyChangeListener(listener);
	}
	
	public Book() {
		super();
	}
	
	public Book(String author, String title, int pageNum, boolean isRead) {
		this();
		this.author = author;
		this.title = title;
		this.pageNum = pageNum;
		this.isRead = isRead;
	}
	public boolean isRead() {
		return isRead;
	}
	public void setRead(boolean isRead) {
		boolean oldIsRead = this.isRead;
		this.isRead = isRead;
		changeSupport.firePropertyChange("isRead", oldIsRead, isRead);	
	}
	public void setAuthor(String author) {
		String oldAuthor = this.author;
		String oldDescription = getDescription();
		this.author = author;
		changeSupport.firePropertyChange("author", oldAuthor, author);
		changeSupport.firePropertyChange("description", oldDescription, getDescription());
	}

	public void setTitle(String title) {
		String oldTitle = this.title;
		String oldDescription = getDescription();
		this.title = title;
		changeSupport.firePropertyChange("title", oldTitle, title);
		changeSupport.firePropertyChange("description", oldDescription, getDescription());
	}

	public void setPageNum(int pageNum) {
		int oldPageNum = this.pageNum;
		this.pageNum = pageNum;
		changeSupport.firePropertyChange("pageNum", oldPageNum, pageNum);
	}

	public void setChangeSupport(PropertyChangeSupport changeSupport) {
		this.changeSupport = changeSupport;
	}

	public String getAuthor() {
		return author;
	}
	public String getTitle() {
		return title;
	}
	public int getPageNum() {
		return pageNum;
	}
	public String getDescription() {
		return String.format("%s : %s", author, title);
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((author == null) ? 0 : author.hashCode());
		result = prime * result + (isRead ? 1231 : 1237);
		result = prime * result + pageNum;
		result = prime * result + ((title == null) ? 0 : title.hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Book other = (Book) obj;
		if (author == null) {
			if (other.author != null)
				return false;
		} else if (!author.equals(other.author))
			return false;
		if (isRead != other.isRead)
			return false;
		if (pageNum != other.pageNum)
			return false;
		if (title == null) {
			if (other.title != null)
				return false;
		} else if (!title.equals(other.title))
			return false;
		return true;
	}
}
